// Define the dimensions of the chart
const margin = {
  top: 10,
  right: 10,
  bottom: 40,
  left: 60,
};
const width = 900 - margin.left - margin.right;
const height = 450 - margin.top - margin.bottom;

// Append SVG element to the chart div
const svg = d3
  .select('#chart')
  .append('svg')
  .attr('width', width + margin.left + margin.right)
  .attr('height', height + margin.top + margin.bottom)
  .append('g')
  .attr(
    'transform',
    `translate(${margin.left},${margin.top})`,
  );

d3.csv(
  'https://raw.githubusercontent.com/Pothugunt1a/Project_DataViz_Database/main/ICRISAT-District%20Level%20Data.csv',
).then(function (data) {
  // Convert production values to numbers
  data.forEach(function (d) {
    d.RICE_PRODUCTION = +d['RICE PRODUCTION (1000 tons)'];
  });

  // Transform data to sum rice production for each state
  const stateData = d3.rollup(
    data,
    (v) => d3.sum(v, (d) => d.RICE_PRODUCTION),
    (d) => d['State Name'],
  );

  console.log('State-wise Data:', stateData);

  // Define x and y scales
  const x = d3
    .scaleBand()
    .domain(Array.from(stateData.keys()))
    .range([0, width])
    .padding(0.3);

  const y = d3
    .scaleLinear()
    .domain([0, d3.max(Array.from(stateData.values()))])
    .range([height, 0]);

  // Define a function to generate unique colors for each state
  const customPurpleGreyColors = [
    '#7b6c92',
    '#b9a0cf',
    '#e1d5e7',
    '#8c8c8c',
    '#b2b2b2',
    '#604e79',
    '#9c89b8',
    '#d5c7e1',
    '#757575',
    '#a6a6a6',
    '#513e65',
    '#8775a2',
    '#c0aed4',
    '#636363',
    '#8c8c8c',
    '#42334e',
    '#7a669a',
    '#b2a0cc',
    '#525252',
    '#757575',
  ];

  const colorScale = d3
    .scaleOrdinal()
    .range(customPurpleGreyColors)
    .domain(Array.from(stateData.keys()));
  // Define a function to generate unique colors for each state
  // const colorScale = d3
  //   .scaleOrdinal(d3.schemeCategory10)
  //   .domain(Array.from(stateData.keys()));

  // Add bars to the chart
  svg
    .selectAll('rect')
    .data(Array.from(stateData.entries()))
    .enter()
    .append('rect')
    .attr('x', (d) => x(d[0]))
    .attr('y', (d) => y(d[1]))
    .attr('width', x.bandwidth())
    .attr('height', (d) => height - y(d[1]))
    .attr('fill', (d) => colorScale(d[0])) // Use the state name to get a color from the scale
    .on('mouseover', function (event, d) {
      // Sum rice production for all districts of the hovered state
      const totalProduction = data
        .filter((entry) => entry['State Name'] === d[0])
        .reduce(
          (sum, entry) => sum + entry.RICE_PRODUCTION,
          0,
        );

      // Round off production value to two decimal places
      const roundedProduction = totalProduction.toFixed(2);

      // Show tooltip on mouseover
      tooltip
        .transition()
        .duration(200)
        .style('opacity', 0.9);
      tooltip
        .html(
          `<strong>${d[0]}</strong><br/>Production: ${roundedProduction} tons`,
        )
        .style('left', event.pageX + 'px')
        .style('top', event.pageY - 28 + 'px');
    })
    .on('mouseout', function () {
      // Hide tooltip on mouseout
      tooltip
        .transition()
        .duration(500)
        .style('opacity', 0);
    });

  // Add x-axis
  svg
    .append('g')
    .attr('transform', `translate(0,${height})`)
    .call(d3.axisBottom(x))
    .selectAll('text')
    .attr('transform', 'rotate(-45)')
    .style('text-anchor', 'end');

  // Add y-axis
  svg.append('g').call(d3.axisLeft(y));
});

// Add tooltip
const tooltip = d3
  .select('body')
  .append('div')
  .attr('class', 'tooltip')
  .style('opacity', 0);
